;;; STEXT.LSP
;;;
;;; 3 November 1994  Guido Haarmans, Neuchatel, Switzerland
;;;
;;; This application searches for a text string within all text entities.
;;; If the string is part of the text entity, then the entity is highlighted
;;; and a red box is drawn around it.
;;; Currently, the search is always case insensitive
;;; The text entities may be in separate User Coordinate Systems.
;;;
;;; If I find the time, I will:
;;;    - update STEXT for R13
;;;    - add support for attributes
;;;
;;; Permission to use distribute, modify or whatever, as long as you
;;; leave my credit in the file.
;;;
;;; This application uses the developer symbol AESC to prevent it from 
;;; clashing with other application.
;;;
;;; Suggestions are welcome, my CompuServe account is 73324,1333

;;; **************************error function *******************************
;; nothing special, only two globals are used
;; no system vaiable is changed
 
(defun aesc_stext_error (msg)
   (princ (strcat "STEXT error:\n"  msg ))
   (aesc_endapp)
   (princ)
)

(defun aesc_endapp ( )
   (setq *error*  aesc_stext_old_error)
   (setq aesc_stext_old_error nil)
   (setq aesc_sset nil)
)

;;; *********************** main program for texts *************************
;; main program
(defun c:stext( / )
    
   ;define own error function
   (setq aesc_stext_old_error  *error*)
   (setq *error* aesc_stext_error)
   
   ; make selection set of all text entities
    (setq aesc_sset (ssget "X" '( (0 . "TEXT"))))  ; aesc_sset is a global
    
    (if aesc_sset
        (aesc_do_stext)
        (aesc_warn "No text entities found in this drawing")
    )

    ;restore error function
    (aesc_endapp)

    (princ)
)


; ****************************** do_stext *********************************
;; This routine cycles through the selection set of text entities

(defun aesc_do_stext ( / str sslen cnt found)    
    
   ; get the string to search for
    (setq str (getstring "\nEnter string to search for: "))

   ; and some values
    (setq sslen (sslength aesc_sset))  ;length of selection set
    (setq cnt 0)                       ; counter
    (setq found 0)                     ; number that matches
    
   ; now walk through the selection set
    (while (< cnt sslen)
        (setq ent (ssname aesc_sset cnt))                   ;get entity name
        (if (aesc_instr str(cdr(assoc 1 (entget ent))) nil) ;look for string
            (progn
                (aesc_high ent)                                               ; highlight and draw box
                (setq found (1+ found))                     ; one more found
            )
        ) 
        (setq cnt (1+ cnt))                                                    ; next entity in selection set
    )
    
    (princ (strcat "\nFound " (itoa found)
                   (if (= found 1) (eval " entity") (eval " entities"))
                   " with string \"" str "\"."))
    (princ)
    
) 

; ******************************* instr ************************************
;; checks if sub is substring of string. Sense is T or nil. 
;; T is for case sensitive

(defun aesc_instr (sub string sense / subl stringl )
    
   ;case senstive or not
    (if (not sense) 
        (setq sub    (strcase sub)
        string (strcase string))
    )
    
    ; length of the strings
    (setq subl (strlen sub)
    stringl (strlen string))
    
    ;initialize some values
    (setq match nil)                 ; sub matches string, then match is T
    (setq in 0)                      ; counter for the position in string
    (setq stop (1+ (- stringl subl))); number of comparisons needed

   ;check it out    
    (while (and (not match) (< in stop) ) 
        (setq in (1+ in))
        (if (= sub (substr string in subl) )  (setq match T) )
    )

    ;return value for this function
    (eval match)
)    

; ******************************* high *************************************
;;  higlights the text entity and draws a red box around it

(defun aesc_high (ent / entl box boxl boxh ang 
                        ang90 evec veclist p1 p2 p3 p4)
    
    ;highlight the entity
    (redraw ent 3)
    
    ; ;now draw a red box around it

    ;get some values
    (setq entl (entget ent))            ; entity list
    (setq box (cadr (textbox entl)))
    (setq boxl (car box))               ; length of text box
    (setq boxh (cadr box))              ; height of tex tbox
    (setq ang (cdr (assoc 50 entl)))    ; angle of text
    (setq ang90 (+ ang (/ pi 2)))       ; angle plus 90 degrees
    (setq evec (cdr (assoc 210 entl ))) ; 3d extrusion vector of entity
    
    ; calculate points (in ESC) of the box to be drawn   
    (setq
        p1  (cdr (assoc 10 entl  ))
        p2  (polar p1 ang boxl)
        p3 (polar p2 ang90 boxh) 
        p4 (polar p1 ang90 boxh)
    )
    
    ; points need to be converted from ESC to current UCS to support 
    ; entities that are not planar to the current UCS
    (setq
        p1 (trans p1  evec 1)
        p2 (trans p2 evec 1)
        p3 (trans p3 evec 1)
        p4 (trans p4 evec 1)
    )
    
    ; grvecs list   
    (setq veclist (list 1 p1 p2 1 p2 p3 1 p3 p4 1 p4 p1))
    
    ; draw the box
    (grvecs veclist)
)

; ****************************** warn **********************************
;; places warning message on screen. Uses (alert) if available, which is
;; from R12 and onwards

(defun aesc_warn (msg /)
    (if alert 
        (alert msg)
        (princ(strcat "\n" msg))
    )
   (princ)
)
;; ******************************* load message *************************************
(prompt "\nSTEXT loaded, enter STEXT to search a string in text entities")
(princ)


